#!/usr/bin/perl
use strict;

# usage:  mrinfo-to-sets.pl mrinfo-data
# Extracts the highest confidence sets from mrinfo responses.

# state
my @alias_sets = ();
my $dst = undef;
my %response_ifaces = ();
my %sets;
my %ifaces;
my $n_sets = 0;

my $cmdline = join(' ', $0, @ARGV);

sub isBogon($)
{
    my ($addr) = @_;
    return
	$addr =~ /0\./ ||                          # this network
	$addr =~ /127\./ ||                        # loopback
	$addr =~ /169\.254\./ ||                   # link local (RFC3330)
	$addr =~ /192\.0\.2\./ ||                  # test net (RFC3330)
	$addr =~ /198\.1[89]\./ ||                 # benchmark (RFC3330)
	$addr =~ /(22[4-9]|23\d)\./ ||             # 224/8 - 239/8 multicast
	$addr =~ /255\./ ||                        # broadcast
	$addr =~ /10\./ ||                         # private - RFC 1918
	$addr =~ /172\.(1[6-9]|2\d|3[01]|)\./ ||   # private - RFC 1918
	$addr =~ /192\.168\./;                     # private - RFC 1918
}

sub endresponse()
{
    if (scalar keys %response_ifaces > 0) {
	my $set = join(' ', sort keys %response_ifaces);
	$sets{$set}++;
	if (defined $ifaces{$dst} && $ifaces{$dst} ne $set) {
	    $ifaces{$dst} = "multiple responses";
	} else {
	    $ifaces{$dst} = $set;
	}
    }

    $dst = undef;
    %response_ifaces = ();
}

while (<>) {
    #print "######## line: $_";
    if (/^\S/) {
	endresponse if (defined $dst);
    }

#    if (/^DVMRP response from (\d+\.\d+\.\d+\.\d+):/) {
#	$dst = $1;
#	$dst = undef;
#    } elsif (/^fail (\d+\.\d+\.\d+\.\d+)/) {
#	$dst = $1;
#	$dst = undef;

    # Handle two slightly different formats:
    #  1. tcpdump: 193.137.55.201 [version 12.2]:
    #  2. mrinfo:  124.214.85.193 (NaN) [version 12.2,prune,mtrace]:
    if (/^(\d+\.\d+\.\d+\.\d+)(?: \(NaN\))?/) {
	#print "# new response\n";
	$dst = $1;
    } elsif (/^ +0\.0\.0\.0 ->/) {
	# ignore
#    } elsif (/^ +(\d+\.\d+\.\d+\.\d+) -> 0\.0\.0\.0/) {
#	# ignore
    } elsif (/^ +(\d+\.\d+\.\d+\.\d+) ->/) {
	#print "# iface line\n";
        my $addr = $1;
	next if isBogon($addr);
	$response_ifaces{$addr}++;
    }
}
endresponse() if (defined $dst);

print "# command line: $cmdline\n";

my $n_sets = 0;
while (my ($set, $ref) = each %sets) {
    my @good = grep {$ifaces{$_} eq $set} split(/ /, $set);
    next if (scalar @good <= 1);
    $n_sets++;
    print "node N$n_sets: ", join(' ', @good), "\n";
}

