#!/usr/bin/perl
use strict;
use Getopt::Long;

my %dsts;
my $dst = undef;
my %include = ();
my $include = undef;

GetOptions(
    "include=s" => \$include)
or die;

if (defined $include) {
    open(INCLUDE, "<", $include) or die "$include: $!\n";
    while (<INCLUDE>) {
	if (/(\d+\.\d+\.\d+\.\d+)/) {
	    $include{$1} = 1;
	}
    }
}

while (<>) {
    $dst = undef if (/^\S/);
    if (/^fail (\d+\.\d+\.\d+\.\d+)/) {
	next if defined $include && !$include{$1};
	$dsts{$ARGV}{$1} = $_ unless defined $dsts{$ARGV}{$1};
    } elsif (/^(\d+\.\d+\.\d+\.\d+) \([^)]*\)/) {
	next if defined $include && !$include{$1};
	$dst = $1;
	$dsts{$ARGV}{$dst} = $';
    } elsif (/^  (\d+\.\d+\.\d+\.\d+) ->/) {
	$dsts{$ARGV}{$dst} .= $_;
    }
}

my $firstfilename = undef;
for my $filename (keys %dsts) {
    if (!defined $firstfilename) {
	$firstfilename = $filename;
	next;
    }
    my $n_probed_in_both = 0;
    my $n_same = 0;
    my $n_fail_both = 0;
    my $n_ok_both = 0;
    my $n_fail_1st = 0;
    my $n_fail_2nd = 0;
    my $n_ok_diff = 0;
    while (my ($dst, $resp) = each %{$dsts{$filename}}) {
	my $firstresp = $dsts{$firstfilename}{$dst};
	next unless defined $firstresp;
	$n_probed_in_both++;
	if ($resp eq $firstresp) {
	    $n_same++;
	} else {
	    $n_ok_diff++ if ($firstresp !~ /^fail/ && $resp !~ /^fail/);
	}
	$n_fail_both++ if ($firstresp =~ /^fail/ && $resp =~ /^fail/);
	$n_ok_both++ if ($firstresp !~ /^fail/ && $resp !~ /^fail/);
	if ($firstresp =~ /^fail/ && $resp !~ /^fail/) {
	    print "fail 1st only: $dst\n";
	    $n_fail_1st++;
	}
	if ($firstresp !~ /^fail/ && $resp =~ /^fail/) {
	    print "fail 2nd only: $dst\n";
	    $n_fail_2nd++;
	}
    }

    print "#### comparing $firstfilename to $filename\n";
    printf "# number of dsts probed in both:       %8d\n", $n_probed_in_both;
    printf "# number of dsts with same result:     %8d (%6.3f%%)\n", $n_same, $n_same/$n_probed_in_both*100;
    printf "# number of dsts that failed in both:  %8d (%6.3f%%)\n", $n_fail_both, $n_fail_both/$n_probed_in_both*100;
    printf "# number of dsts ok in both:           %8d (%6.3f%%)\n", $n_ok_both, $n_ok_both/$n_probed_in_both*100;
    printf "# number of dsts ok but different:     %8d (%6.3f%%)\n", $n_ok_diff, $n_ok_diff/$n_probed_in_both*100;
    printf "# number of dsts that failed 1st only: %8d (%6.3f%%)\n", $n_fail_1st, $n_fail_1st/$n_probed_in_both*100;
    printf "# number of dsts that failed 2nd only: %8d (%6.3f%%)\n", $n_fail_2nd, $n_fail_2nd/$n_probed_in_both*100;
}

