#!/usr/bin/perl
use strict;

# config
my $sort_response = 1;

# state
my $dst = undef;
my $meta = undef;
my @lines = ();
my %response_ifaces = ();
my %responses;
my %ifaces;
my %dsts;
my $n_other = 0;
my $n_empty_resp = 0;
my @similarities = ();

sub isBogon($)
{
    my ($addr) = @_;
    return
	$addr =~ /0\./ ||                          # this network
	$addr =~ /127\./ ||                        # loopback
	$addr =~ /169\.254\./ ||                   # link local (RFC3330)
	$addr =~ /192\.0\.2\./ ||                  # test net (RFC3330)
	$addr =~ /198\.1[89]\./ ||                 # benchmark (RFC3330)
	$addr =~ /(22[4-9]|23\d)\./ ||             # 224/8 - 239/8 multicast
	$addr =~ /255\./ ||                        # broadcast
	$addr =~ /10\./ ||                         # private - RFC 1918
	$addr =~ /172\.(1[6-9]|2\d|3[01]|)\./ ||   # private - RFC 1918
	$addr =~ /192\.168\./;                     # private - RFC 1918
}

sub response_similarity($$)
{
    my ($r1, $r2) = @_;
    return 1 if ($r1 eq $r2);
    my $ref1 = $responses{$r1};
    my $ref2 = $responses{$r2};
    my %r1ifaces = ();
    my $shared = 0;
    my $total = 0;
    # print "## r1 ifaces: ", $ref1->{ifaces} ? join(', ', @{$ref1->{ifaces}}) : "undef", "\n";
    # print "## r2 ifaces: ", $ref2->{ifaces} ? join(', ', @{$ref2->{ifaces}}) : "undef", "\n";
    for my $iface (@{$ref1->{ifaces}}) {
	$r1ifaces{$iface} = 1;
	$total++
    }
    for my $iface (@{$ref2->{ifaces}}) {
	if ($r1ifaces{$iface}) {
	    $shared++;
	} else {
	    $total++;
	}
    }
    # print "## similarity = $shared/$total\n";
    return ($total == 0 ? 0 : $shared/$total) . " ($shared/$total)";
}

sub endresponse()
{
    if (scalar keys %response_ifaces == 0) {
	$n_empty_resp++;
    } else {
	my $response = $sort_response ? join('', sort @lines) : join('', @lines);
	$responses{$response}{count}++;
	my $resp_obj = $responses{$response};
	if ($resp_obj->{count} == 1) {
	    $resp_obj->{meta} = $meta;
	    @{$resp_obj->{ifaces}} = sort keys %response_ifaces;
	    # print "#### ifaces: ", join(', ', @{$resp_obj->{ifaces}}), "\n";
	    for my $iface (keys %response_ifaces) {
		next if (isBogon($iface));
		push @{$ifaces{$iface}}, $response;
	    }
	}
	$dsts{$dst}{n_raw_response}++;
	if ($dsts{$dst}{response}[0] ne $response) {
	    $dsts{$dst}{n_response}++;
	    my $n = $dsts{$dst}{n_response};
	    if ($n > 1) {
		my $sim = response_similarity($dsts{$dst}{response}[0], $response);
		my $metasim =
		    $responses{$dsts{$dst}{response}[0]}{meta} eq $responses{$response}{meta} ?
		    "same" : "different";
		push @similarities, 0+$sim;
		print "## $dst similarity of response 1 vs $n: $sim; meta: $metasim\n";
		if ($dsts{$dst}{n_response} == 2) {
		    print "## $dst response #1:\n";
		    print $dsts{$dst}{response}[0];
		}
		print "## $dst response #$n:\n";
		print $response;
		print "##\n";
	    }
	    push @{$dsts{$dst}{response}}, $response;
	}
	push @{$responses{$response}{dsts}}, $dst;
	#print "########\n";
	#print "response:\n$response";
	#print "dst: $dst\n";
	#print "ifaces: ", join(', ', keys %response_ifaces), "\n";
    }

    @lines = ();
    $dst = undef;
    $meta = undef;
    %response_ifaces = ();
}

sub histogram($$)
{
    my ($dataref, $label) = @_;
    print "# histogram of $label\n";
    for (my $i = 0; $i < scalar @$dataref; $i++) {
	next unless ($dataref->[$i]);
	printf "%4d %6d\n", $i, $dataref->[$i];
    }
}

while (<>) {
    #print "######## line: $_";
    if (/^\S/) {
	endresponse if (defined $dst);
    }

#    if (/^DVMRP response from (\d+\.\d+\.\d+\.\d+):/) {
#	$dst = $1;
#	$dsts{$dst}{n_error}++;
#	push @{$dsts{$dst}{response}}, $_;
#	$dst = undef;
#    } elsif (/^fail (\d+\.\d+\.\d+\.\d+)/) {
#	$dst = $1;
#	$dsts{$dst}{n_fail}++;
#	push @{$dsts{$dst}{response}}, $_;
#	$dst = undef;

    if (/^(\d+\.\d+\.\d+\.\d+) \([^)]*\)/) {
	#print "# new response\n";
	$dst = $1;
	@lines = ($');
	$meta = $';
    } elsif (/^ +0\.0\.0\.0 ->/) {
	# ignore
#    } elsif (/^ +(\d+\.\d+\.\d+\.\d+) -> 0\.0\.0\.0/) {
#	# ignore
    } elsif (/^ +(\d+\.\d+\.\d+\.\d+) ->/) {
	#print "# iface line\n";
	push @lines, $_;
	$response_ifaces{$1}++;
    } else {
	$n_other++;
    }
}
endresponse() if (defined $dst);

print "number of unique non-bogon ifaces: ", scalar keys %ifaces, "\n";
print "number of unique destinations: ", scalar keys %dsts, "\n";
print "number of empty responses: ", $n_empty_resp, "\n";
print "number of unique non-empty responses: ", scalar keys %responses, "\n";
print "number of other lines: ", $n_other, "\n";

my @n_response = ();
my @n_raw_response = ();
#my @n_error = ();
#my @n_fail = ();
my @n_self = ();
while (my ($dst, $ref) = each %dsts) {
    $n_response[$ref->{n_response}]++;
    $n_raw_response[$ref->{n_raw_response}]++;
    #$n_error[$ref->{n_error}]++;
    #$n_fail[$ref->{n_fail}]++;
    my $response = $ref->{response}[0];
    my $self = grep($dst eq $_, @{$responses{$response}{ifaces}});
    @n_self[$self]++;
}
histogram(\@n_raw_response, "number of raw responses per destination");
histogram(\@n_response, "number of unique responses per destination");
#histogram(\@n_error, "number of errors per destination");
#histogram(\@n_fail, "number of fails per destination");
histogram(\@n_self, "number of destinations whose response included the destination");

@similarities = sort @similarities;
print "# similarities: (", scalar @similarities, ")\n", join("\n", @similarities), "\n";

my @appearances = ();
my @n_conflicting_resp = ();
my $n_nonbogon_ifaces_without_resp = 0;
my $n_nonbogon_ifaces_with_resp = 0;
while (my ($iface, $responses) = each %ifaces) {
    $appearances[scalar @{$responses}]++;

    my $dstref = $dsts{$iface};
    if (!defined $dstref) {
	$n_nonbogon_ifaces_without_resp++;
    } else {
	$n_nonbogon_ifaces_with_resp++;
	my %dst_resp = ();
	my $n = 0;
	for my $response (@{$dsts{$iface}{response}}) { $dst_resp{$response} = 1; }
	for my $response (@{$responses}) {
	    $n++ unless ($dst_resp{$response});
	}
	$n_conflicting_resp[$n]++;
    }
}
histogram(\@appearances, "number of unique responses in which each non-bogon iface appeared");
print "# number of non-bogon ifaces without responses: $n_nonbogon_ifaces_without_resp\n";
print "# number of non-bogon ifaces with responses: $n_nonbogon_ifaces_with_resp\n";
histogram(\@n_conflicting_resp, "responses including an iface that disagreed with the iface's response");

my @iface_histogram = ();
my @count_histogram = ();
while (my ($response, $ref) = each %responses) {
    $iface_histogram[scalar @{$ref->{ifaces}}]++;
    $count_histogram[$ref->{count}]++;
}
histogram(\@count_histogram, "number of appearances per response");
histogram(\@iface_histogram, "number of ifaces per response");
