#! /usr/local/bin/perl
# usage: parse-dns.pl {dnsdump}... >sets
# The dnsdump file should contain lines with timestamp, ipaddr, hostname.

use strict;

print "# command line: $0 ", join(" ", @ARGV), "\n";

my %routeraddrs;
my %addrs;
my $changed = 0;
my $bogons = 0;

sub isBogon($) {
    $_ = shift;
    return
	/^0\./ ||
	/^127\./ ||
	/^169\.254\./ ||
	/^192\.0\.2\./ ||
	/^198\.1[89]\./ ||
	/^2(2[4-9]|3[0-9])\.\./ ||
	/^255\./ ||
	/^10\./ ||
	/^172\.(1[6-9]|2[0-9]|3[01])\./ ||
	/^192\.168\./;
}

sub Load() {
    while(<>) {
	next if (/^#/);
	next if (/^FAILED/);
	my ($time, $ipaddr, $hostname);
	my @F = split;
	if ($F[0] =~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/) {
	    $ipaddr = $F[0];
	    $time = $F[1];
	    $hostname = $F[2];
	} elsif ($F[1] =~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/) {
	    $time = $F[0];
	    $ipaddr = $F[1];
	    $hostname = $F[2];
	} else {
	    die "unknown format at $.\n";
	}
	if (isBogon($ipaddr)) {
	    $bogons++;
	    next;
	}

	my ($iface, $router) = ($hostname =~ /^([a-z][-a-z0-9]*[0-9](?:\.\d+)?)\.(([a-z]+\d\d(?:\.b\d{6}-\d+)?|[a-z]-root|res-gw|route-server)\.(hq|[a-z]{3})\d\d\.(atlas|hades)\.cogentco\.com)$/);
	if (defined $router) {
	    # print STDERR "ipaddr=\"$ipaddr\" iface=\"$iface\" router=\"$router\"\n";
	    $addrs{$ipaddr}{count}++;
	    $addrs{$ipaddr}{routers}{$router}++;
	    if ($time > $addrs{$ipaddr}{time}) {
		$addrs{$ipaddr}{time} = $time;
		$addrs{$ipaddr}{router} = $router;
	    }
	} else {
	    print STDERR "warning: $ipaddr has unparsable hostname \"$hostname\"\n";
	    print "# warning: $ipaddr has unparsable hostname \"$hostname\"\n";
	}
    }

    while (my ($ipaddr, $ref) = each %addrs) {
	if (scalar keys %{$ref->{routers}} > 1) {
	    $changed++;
	    print STDERR "warning: $ipaddr found on multiple routers: ", join(", ", keys %{$ref->{routers}}), "\n";
	    print "# warning: $ipaddr found on multiple routers: ", join(", ", keys %{$ref->{routers}}), "\n";
	} else {
	    push @{$routeraddrs{$ref->{router}}}, $ipaddr;
	}
    }
}

Load();
my @routers = keys %routeraddrs;

printf "# routers:                        %9d\n", scalar(@routers);
printf "# bogons:                         %9d\n", $bogons;
printf "# addresses:                      %9d\n", scalar keys %addrs;
printf "# addrs that appear 1 time:       %9d\n", scalar(grep {$addrs{$_}{count} == 1} keys %addrs);
printf "# addrs that appear >1 time:      %9d\n", scalar(grep {$addrs{$_}{count} > 1} keys %addrs);
printf "# addrs that appear on >1 router: %9d\n", $changed;

my $nodenum = 0;
for my $router (sort {$a <=> $b} @routers) {
    print "# name: $router\n";
    print join("\n", sort @{$routeraddrs{$router}}), "\n";
}

# writeSets();
